
//RoomManager provides methods for communicating with the server regarding the configuration of rooms and displays
function RoomManager(rooms, urls) {
    return {
        getRooms: function () {
            return rooms;
        },
        getRoomById: function (id) {
            for (var i = 0; i < rooms.length; i++) {
                if (rooms[i].Id == id) {
                    return rooms[i];
                }
            }
            return null;
        },
        doesRoomWithAddressExist: function (address) {
            for (var i = 0; i < rooms.length; i++) {
                if (rooms[i].Address === address) {
                    return true;
                }
            }

            return false;
        },
        associateRoomWithDisplay: function (roomId, screenId, fn) {
            return $.post(urls.associateUrl, { roomId: roomId, screenId: screenId }, function (data) {
                if (fn) {
                    fn(data);
                }
            });
        },
        deassociateRoomFromDisplay: function (roomId, screenId, fn) {
            return $.post(urls.deassociateUrl, { roomId: roomId, screenId: screenId }, function (data) {
                if (fn) {
                    fn(data);
                }
            });
        },
        updateDisplayName: function (roomId, displayName, fn) {
            $.post(urls.updateDisplayNameUrl, { roomId: roomId, displayName: displayName }, function (data) {
                //Invoke callback if present
                if (fn) {
                    fn(data);
                }
            });
        },
        save: function (roomData, fnSaved) {
            $.post(urls.saveRoomUrl, roomData, function (data) {
                if (data.Error) {
                    alert(data.Error);
                    return;
                }
                //Invoke callback if present
                if (fnSaved) {
                    fnSaved();
                }
            });
        },
        loadRoomStatuses: function (fnLoaded) {
            var statusLoader = function loader(callback) {
                $.post(urls.roomStatusesUrl, {}, function (data) {
                    if (data.Available) {
                        callback(data.Statuses);
                    }
                    else {
                        // Recurse until statuses become available 
                        setTimeout(function () {
                            loader(callback);
                        }, 1000);
                    }
                });
            };
            statusLoader(fnLoaded);
        },
        resolve: function (address, fn) {
            $.post(urls.resolveRoomUrl, { address: address }, function (data) {
                fn(data);
            });
        }
    };
}

function ScreenManager() {
    var jTableHeader = $('#displayConfiguration thead');
    var jScreenRows = $('th', jTableHeader);

    var screens = [];
    jScreenRows.each(function () {
        var jRow = $(this);
        var screenData = jRow.data("screenData");
        if (screenData) {
            screens[screenData.Id] = screenData;
        }
    });

    return {
        getScreenById: function (id) {
            return screens[id];
        }
    };
    
}

//RoomFinder encapsulates the UI and logic for finding a room in Exchange
function RoomFinder(roomManager, fnFoundRoomData) {
    var roomFinder = $('#roomFinder');
    var mailBoxAddressInput = $('#roomFinder input[name="Address"]');
    //"Enter" key behaves as submit
    mailBoxAddressInput.keyup(function (event) {
        if (event.keyCode == 13) {
            $("#searchForMailboxBtn").click();
        }
    });

    var startRoomFinding = function () {
        $('#feedbackDiv').hide();
        $("#buttonPanel").hide();
        $("#roomFinder").show();
        mailBoxAddressInput.select();
    };

    var endRoomFinding = function () {
        $('#feedbackDiv').hide();
        $('#buttonPanel').show();
    };

    var searchResultUI = (function () {
        var okButtonMarkup = '<input id="okBtn" type="button" value="OK"/>';
        var importButtonMarkup = '<input id="importRoomBtn" type="button" value="Import room"/>';
        var cancelButtonMarkup = '<input id="cancelImportBtn" type="button" value="Cancel"/>';

        function foundRoomMarkup(room) {
            return 'Found room:<br/>Mailbox address: ' + room.Address + ' (Name: ' + room.NameInDataSource + ')<br/><br/>';
        }

        return {
            showResolveError: function (data) {
                ShowResponse('Could not resolve the specified mailbox address: ' + data.Error, true);
                $('#buttonPanel').show();
            },
            showRoomAlreadyImported: function (room) {
                ShowResponse(foundRoomMarkup(room) + 'This room has already been imported.<br/>' + okButtonMarkup);
                $('#okBtn').click(function () {
                    endRoomFinding();
                }).focus();
            },
            showAskToImport: function (room) {
                ShowResponse(foundRoomMarkup(room) + importButtonMarkup + cancelButtonMarkup);
                $("#importRoomBtn").click(function () {
                    endRoomFinding();
                    fnFoundRoomData({ Id: 0, Address: room.Address, NameInDataSource: room.NameInDataSource });
                }).focus();

                $("#cancelImportBtn").click(function () {
                    endRoomFinding();
                });
            }
        };
    })();

    $("#searchForMailboxBtn").click(function () {
        var mailBoxAdress = mailBoxAddressInput.val();
        if (!mailBoxAdress) {
            return;
        }

        roomFinder.hide();

        ShowProgress("Searching for " + mailBoxAdress + " in Microsoft Exchange...");
        roomManager.resolve(mailBoxAdress, function (data) {
            // Show any errors received
            if (data.Error) {
                searchResultUI.showResolveError(data);
                return;
            }

            // Check to see if this room has already been imported
            if (roomManager.doesRoomWithAddressExist(data.Address)) {
                searchResultUI.showRoomAlreadyImported(data);
                return;
            }

            // The room hasn't been imported yet - ask if the user wants to import it
            searchResultUI.showAskToImport(data);
        });
    });

    $("#cancelSearchForMailboxBtn").click(function () {
        roomFinder.hide();
        endRoomFinding();
    });

    return {
        show: function () {
            startRoomFinding();
        }
    };

}

//RoomTable encapsulates the room/display table grid
function RoomTable() {
    var jTableBody = $('#displayConfiguration tbody');
    var jRoomRows = $('tr', jTableBody);
    var roomClickHandler = function () { };
    var screenAssociationChangedHandler = function () { };
    var roomRows = [];
    var roomColumns = [];

    //TODO: Collapse this loop with the "Configuration of displays" loop
    jRoomRows.each(function () {
        var jRow = $(this);
        var roomId = jRow.data("roomId");
        var roomColumn = jRow.find('td:nth-child(1)');
        roomRows[roomId] = jRow;
        roomColumns[roomId] = roomColumn;

        roomColumn.find("a").click(function () {
            roomClickHandler(jRow.data("roomData"));
        });
    });

    // Configuration of displays
    $("input:checkbox", $('#displayConfiguration tbody')).click(function () {
        var selectedRoomId = $(this).closest("tr").data("roomId");
        var selectedScreenId = $(this).closest("td").data("screenId");
        screenAssociationChangedHandler(selectedRoomId, selectedScreenId, this.checked);
        //Toggle Edit link (if present)
        //$(this).parent().next("td").find("span").toggle(this.checked);
    });

    return {
        extractRoomData: function () {
            var rooms = [];
            jRoomRows.each(function () {
                var roomData = $(this).data("roomData");
                rooms.push(roomData);
            });
            return rooms;
        },
        setRoomClickHandler: function (fn) {
            roomClickHandler = fn;
        },
        setScreenAssociationChangedHandler: function (fn) {
            screenAssociationChangedHandler = fn;
        },
        setRoomStatusError: function (roomId, error) {
            var jStatusIndicator = roomColumns[roomId].find('span');
            jStatusIndicator.addClass('screenIdle').attr('title', error);
        },
        setRoomDisplayName: function (roomId, displayName) {
            var jDisplayLabel = roomRows[roomId].find('span.displayNameLabel');
            jDisplayLabel.html(displayName);
        }
    };
}

//ScreenEditor encapuslates UI and logic for creating a new screen
function ScreenEditor(createPath, header) {
    var saveUrl = createPath;
    function save(screenName, fn) {
        if (!screenName) {
            alert('Name cannot be empty...');
            return;
        }
        $.post(saveUrl, { name: screenName }, function (data) {
            if (data.Error) {
                alert(data.Error);
                return;
            }
            fn(data);
        });
    }

    return {
        show: function () {
            $("<div></div>")
                .append("<label>Name</label>")
                .append($("<input class='newScreenName'>", { type: "text", id: "newScreenName"}))
                .dialog({ modal: true,
                    width: 400,
                    title: header,
                    open:function() {
                        $(this).parents(".ui-dialog:first").find(".ui-dialog-titlebar-close").remove();
                    },
                    buttons: {
                        Cancel: function () {
                            $(this).dialog('close').remove();
                        },
                        Save: function () {
                            var screenName = $(".newScreenName").first().val();
                            var jDialog = $(this);
                            save(screenName, function () {
                                jDialog.dialog('close').remove();
                                RefreshAction();
                            });
                        }
                    }
                });
        }
    };
}
