﻿/*!
* TimePicker Plugin
* 
* Copyright 2009-2011 Pronestor Aps
* Author: Kasper Frank
* 
*/
(function ($) {
    var TimePicker = new function () {
        var today = dateOnly(new Date());
        var values = []; //for storing option values

        // returns only the date part of the supplied Date
        function dateOnly(date) {
            return new Date(date.getFullYear(), date.getMonth(), date.getDate());
        }

        function appendOption(obj, value, text) {
            var settings = obj.data('settings');

            //generate appropriate am/pm designator if any
            var ampm_designator = "";
            if (settings.ampm)
                ampm_designator = getIntValue(value) >= 1200 ? settings.pmDesignator : settings.amDesignator;

            //should the option be selected?
            var selected = getIntValue(value) == settings.time ? 'selected="selected" ' : '';
            if (settings.amPmDesignatorFront)
                obj.append('<option ' + selected + 'value="' + value + '">' + ampm_designator + ' ' + text + '</option>');
            else
                obj.append('<option ' + selected + 'value="' + value + '">' + text + ' ' + ampm_designator + '</option>');
        }

        function isValueInsideBounds(intValue, settings) {
            //date is outside bounds
            if (settings.date < settings.minDate && settings.date > settings.maxDate)
                return false;

            // the date is both different from minDate and maxDate and should be added
            if (settings.date.valueOf() !== settings.minDate.valueOf() && settings.date.valueOf() !== settings.maxDate.valueOf())
                return true;

            // the date is the same as both minDate and maxDate and therefor needs to satisfy both minTime and maxTime			
            if (settings.minDate.valueOf() === settings.maxDate.valueOf())
                return intValue >= getIntValue(settings.minTime) && intValue <= getIntValue(settings.maxTime);

            //if the date is equal to the minDate, the minTime constraint must be satisfied
            if (settings.date.valueOf() === settings.minDate.valueOf())
                return intValue >= getIntValue(settings.minTime);
            //the date is equal to the maxDate and the maxTime constraint must be satisfied
            else
                return intValue <= getIntValue(settings.maxTime);
        }

        function toHourText(value, settings) {
            // make value a string add pad it such that its 4 characters long, e.g. 748 -> '0748' 
            value += '';
            if (value.length < 4) {
                while (value.length < 4) {
                    value = "0" + value;
                }
            }
            var hourPart = getIntValue(value.substring(0, 2));
            var hourText = settings.ampm ? (hourPart <= 12 ? (hourPart == 0 ? 12 : hourPart) : (hourPart - 12)) : hourPart;
            return hourText + ":" + value.substring(2, value.length);
        }

        function getHours(value) {
            // make value a string add pad it such that its 4 characters long, e.g. 748 -> '0748' 
            value += '';
            if (value.length < 4) {
                while (value.length < 4) {
                    value = "0" + value;
                }
            }
            var hourPart = getIntValue(value.substring(0, 2));
            return hourPart;
        }

        function getMinutes(value) {
            // make value a string add pad it such that its 4 characters long, e.g. 748 -> '0748' 
            value += '';
            if (value.length < 4) {
                while (value.length < 4) {
                    value = "0" + value;
                }
            }
            var minutesPart = getIntValue(value.substring(2, value.length));
            return minutesPart;
        }

        function getIntValue(value) {
            return parseInt(value, 10);
        }

        // Populates the element (obj)  with select options
        function populate(obj) {
            var settings = obj.data('settings');
            obj.html("");
            var minTime = getIntValue(settings.minTime);
            var maxTime = getIntValue(settings.maxTime);
            $.each(values, function (index, val) {
                var value = getIntValue(val.value);
                // add option if it satisfies min and max constraints
                if (isValueInsideBounds(value, settings)) {
                    if (index > 0 && index < values.length) {
                        if (getIntValue(values[index - 1].value) < minTime && minTime < value) {
                            appendOption(obj, minTime, toHourText(minTime, settings));
                        }

                        //also add selected time if it's not included in the intervals
                        var selectedtime = getIntValue(settings.time);
                        if (selectedtime !== minTime && selectedtime !== maxTime && getIntValue(values[index - 1].value) < selectedtime && selectedtime < value) {
                            appendOption(obj, selectedtime, toHourText(selectedtime, settings));
                        }
                    }
                    appendOption(obj, val.value, val.text);
                    // add maxTime as an option if it hasn't already been added and the next value is greater than maxTime (and if maxTime is inside the select day bounds)
                    if (getIntValue(val.value) !== maxTime && index + 1 < values.length - 1 && getIntValue(values[index + 1].value) > maxTime && settings.date.valueOf() === settings.maxDate.valueOf()) {
                        appendOption(obj, maxTime, toHourText(maxTime, settings));
                    }
                }
            });
        }
        return {
            init: function (options) {
                var defaults = {
                    minTime: "0000",
                    maxTime: "2359",
                    minDate: today,
                    maxDate: today,
                    date: today,
                    time: -1,
                    step: 15,
                    ampm: false,
                    hasChanged: false,
                    amDesignator: "",
                    pmDesignator: "",
                    amPmDesignatorFront: false,
                    onchange: function (oldtime, newtime) { }
                };

                //"merge" default settings with supplied options
                var settings = $.extend(defaults, options);
                settings.date = dateOnly(settings.date);
                $(this).data('settings', settings);
                var hiddenId = "#" + $(this).attr('id') + "hidden";
                var hiddenEl = $(hiddenId);
                if (hiddenEl.length > 0) {
                    hiddenEl.data('settings', settings);
                }

                values = [];
                //fill all potential select options
                for (i = 0; i < 24; i++) {
                    var hourText = settings.ampm ? (i <= 12 ? (i == 0 ? 12 : i) : (i - 12)) : i;

                    values = values.concat([{ value: i + "00", text: hourText + ":00"}]);
                    if (settings.step == 15) {
                        values = values.concat([{ value: i + "15", text: hourText + ":15"}]);
                        values = values.concat([{ value: i + "30", text: hourText + ":30"}]);
                        values = values.concat([{ value: i + "45", text: hourText + ":45"}]);
                    }
                    else if (settings.step == 30) {
                        values = values.concat([{ value: i + "30", text: hourText + ":30"}]);
                    }
                }

                $(this).change(function () {
                    console.log("time change");
                    CheckForHiddenSettings($(this));
                    var s = $(this).data('settings');
                    s.hasChanged = true;
                    var newTime = $(this).val();
                    s.onchange(s.time, newTime);
                    s.time = newTime;
                    $(this).data('settings', s);
                });

                return this.each(function () {
                    var obj = $(this);
                    // populate the select item with options
                    populate(obj);
                });
            },
            setDate: function (newDate, newTime) {
                CheckForHiddenSettings($(this));
                var settings = $(this).data('settings');
                return this.each(function () {
                    if (settings) {
                        settings.date = dateOnly(newDate);
                        populate($(this));
                    }
                });
            },
            hasChanged: function () {
                CheckForHiddenSettings($(this));
                return $(this).data('settings').hasChanged;
            },
            getDateTime: function () {
                var me = $(this);
                if (!me.data('settings'))
                    return null;
                var date = dateOnly(me.data('settings').date);
                var time = me.val();
                date.setHours(getHours(time));
                date.setMinutes(getMinutes(time));
                return date;
            }
        };

        function CheckForHiddenSettings(el) {
            var s = el.data('settings');
            if (s == null) {
                var hiddenId = "#" + el.attr('id') + "hidden";
                var hiddenEl = $(hiddenId);
                if (hiddenEl.length > 0) {
                    el.data('settings', hiddenEl.data('settings'));
                }
            }
        }
    } ();
    $.fn.extend({
        TimePicker: TimePicker.init,
        TimePickerSetDate: TimePicker.setDate,
        TimePickerGetDateTime: TimePicker.getDateTime,
        TimePickerHasChanged: TimePicker.hasChanged
    });
})(jQuery);