/**
 * @typedef {object} DurationInput
 * @property {number} duration - Duration of the meeting in minutes.
 * @property {number} maxDuration - Maximum duration of the meeting in minutes.
 * @property {number} minDuration - Minimum duration of the meeting in minutes.
 * @property {date} startTime - Start date and time of the meeting.
 */

/**
 * @typedef {object} ValidationResult
 * @property {number} durationLeft - Duration left in minutes.
 * @property {(string|undefined)} message - Error message. Defined if success is false.
 * @property {boolean} success - True if the duration is valid.
 */

function ScreenApi() {
  var screenApiValidation = new ScreenApiValidation();

  this.initialize = function () {
    if (ScreenModule.data.isRoomScreen()) {
      // confirm
      var current = ScreenModule.data.getCurrent();
      this.setupConfirmFor(current);

      var next = ScreenModule.data.getNext();
      this.setupConfirmFor(next);
    }
  };

  this.activateIdle = function (callback) {
    $(document).on("screenDisplay:idle", function () {
      if (callback) {
        callback();
      }
    });
  };

  this.hideInteractivity = function () {
    // console.log($(":data(displayscreenvisible)"));
    // TODO: does not work in yellow mode
    $(":data(displayscreenvisible)").each(function () {
      $(this).data("displayscreenvisible", false);
    });
  };

  // buttons
  // pincode
  // TODO: have json input like:
  // have 4 events, onShow, onHide, action, onBeforeShow
  // have 1 prop condition that returns "onShow", "onHide", "action" and handles events by that
  // if onShow is ommitted call action, but always and only prompt pincode onShow
  // if onHide is ommitted call action, without pincode promt
  this.activateInteractivity = function (opt, callback) {
    var options = $.extend({ handlebars: true }, opt);
    if (!options.trigger) {
      options.trigger = "click";
    }

    if (options.useLive) {
      console.log("using live on:" + options.selector);
      $(options.selector).live(options.trigger, function (e) {
        interativityHandler(options, e, $(this));
      });
    } else {
      $(document).off(options.trigger, options.selector);
      $(document).on(options.trigger, options.selector, function (e) {
        interativityHandler(options, e, $(this));
      });
    }
  };

  function interativityHandler(options, e, el) {
    console.log(options.selector);

    e.preventDefault();
    e.stopPropagation();
    $(document).trigger("screenDisplay:general", [options.selector + ":" + e.type]);
    // before this event
    if (options.onBeforeShow && !$(options.selector).data("displayscreenvisible")) {
      options.onBeforeShow(e, el);
    }

    // directAction
    if (options.action) {
      ScreenModule.pincode.addPincodeWatch(
        function (accepted) {
          if (accepted) {
            options.action(el, e);
          }
        },
        options.noPincode,
        options.confirm,
        options.handlebars
      );

      return;
    }

    // if show hide elements
    console.log($(options.selector).data("displayscreenvisible"));
    if ($(options.selector).data("displayscreenvisible")) {
      $(options.selector).data("displayscreenvisible", false);
      options.onHide(el, e);
    } else {
      ScreenModule.pincode.addPincodeWatch(
        function (accepted) {
          if (accepted) {
            if (!options.bypassVisibility) {
              $(options.selector).data("displayscreenvisible", true);
            }
            if (options.onShow) {
              options.onShow(el, e);
            }
          } else {
            if (callback) {
              callback(accepted);
            }
          }
        },
        options.noPincode,
        options.confirm,
        options.handlebars
      );
    }
  }

  function hasOnTouchEvent() {
    return "ontouchstart" in document.documentElement;
  }

  this.setupConfirmFor = function (item) {
    if (!PRONESTORDISPLAY.events) {
      PRONESTORDISPLAY.events = {};
    }
    if (!PRONESTORDISPLAY.events.cancelAttached) {
      PRONESTORDISPLAY.events.cancelAttached = [];
    }
    if (item && ScreenModule.data.itemNeedsConfirmation(item) && !isCancelAttached(item.MeetingId)) {
      if (!ScreenModule.data.itemIsConfirmedOnScreen(item)) {
        attachCancelEvent(item.MeetingId);
        setTimeout(function () {
          if (!ScreenModule.data.itemIsConfirmedOnScreen(item)) {
            var cancelOk = isCancelOk(item.MeetingId);
            if (cancelOk) {
              removeCancelEvent(item.MeetingId);
              new ScreenApi().cancel(item.MeetingId);
            }
          } else {
            console.log("removing cancel: " + item.MeetingId);
            removeCancelEvent(item.MeetingId);
          }
        }, item.RemainingConfirmationSeconds * 1000);
      }
    }
  };

  function isCancelAttached(meetingId) {
    if (!cancelEventsDefined()) {
      return false;
    }
    var finding;
    $.each(PRONESTORDISPLAY.events.cancelAttached, function (index, value) {
      if (value == meetingId) {
        finding = meetingId;
      }
    });
    if (finding) {
      return true;
    } else {
      return false;
    }
  }
  function attachCancelEvent(meetingId) {
    if (!cancelEventsDefined()) {
      return;
    }
    PRONESTORDISPLAY.events.cancelAttached.push(meetingId);
  }
  function removeCancelEvent(meetingId) {
    if (!cancelEventsDefined()) {
      return;
    }
    var index = PRONESTORDISPLAY.events.cancelAttached.indexOf(meetingId);
    if (index >= 0) {
      PRONESTORDISPLAY.events.cancelAttached.splice(index, 1);
    }
  }
  function findCancel(meetingId) {
    return meetingId == this;
  }

  function cancelEventsDefined() {
    return PRONESTORDISPLAY && PRONESTORDISPLAY.events && PRONESTORDISPLAY.events.cancelAttached;
  }

  this.getPrimaryElementData = function () {
    if (ScreenModule.data.isAreaScreen()) {
      var f = ScreenModule.data.getFocusedRoom();
      return f;
    } else {
      return ScreenModule.data.getCurrent();
    }
  };

  function isCancelOk(id) {
    // TODO: Implement if needed.
    return true;
  }

  /**
   * Adds `duration` minutes to the CurrentDuration of the current item. If duration is negative, the minutes are subtracted.
   * @param {number} duration - The duration in minutes to add to the current duration.
   */
  this.updateTime = function (duration) {
    var data = ScreenModule.data;
    var tempDuration = data.getCurrent().CurrentDuration + duration;

    /** @type {DurationInput} */
    var durationInput = {
      duration: tempDuration,
      maxDuration: data.getMaxDuration(),
      minDuration: data.getMinDuration(),
      startTime: data.isFree() ? data.getCurrentTime() : data.getCurrent().PeriodDateTimeRange.Start,
    };

    var validationResult = screenApiValidation.validateBook(durationInput);
    if (!validationResult.success) {
      durationInput.duration = data.getCurrent().CurrentDuration;
      var result = ScreenModule.data.getNewDurationStatus(durationInput);
      result.message = validationResult.message;
      return result;
    } else {
      data.getCurrent().CurrentDuration = data.getCurrent().CurrentDuration + duration;
    }

    $(document).trigger("screenDisplay:debug:duration", [data.getCurrentDuration()]);
    return data.getNewDurationStatus(durationInput);
  };

  this.initializeTime = function () {
    var data = ScreenModule.data;
    data.getCurrent().CurrentDuration = this.getInitialDuration();
    var durationInput = {
      duration: data.getCurrent().CurrentDuration,
      maxDuration: data.getMaxDuration(),
      minDuration: data.getMinDuration(),
      startTime: data.isFree() ? data.getCurrentTime() : data.getCurrent().PeriodDateTimeRange.Start,
    };
    $(document).trigger("screenDisplay:debug:duration", [data.getCurrentDuration()]);
    return data.getNewDurationStatus(durationInput);
  };

  this.getInitialDuration = function () {
    var current = ScreenModule.data.getCurrent();
    var totalMinutes = 0;
    var currentHours = current.HoursLeft;
    var currentMinutes = current.MinutesLeft;

    if (ScreenModule.data.isFree()) {
      totalMinutes = ScreenModule.data.getDefaultDuration();
    } else {
      totalMinutes = ScreenModule.time.getDiffMinutes(
        current.PeriodDateTimeRange.Start,
        current.PeriodDateTimeRange.End
      );
    }

    return totalMinutes;
  };

  // interaction

  /**
   * Books the room.
   * @param {function} callback Callback function that is parsed to both `validateTime` and `executeCommand`.
   * @returns {(undefined|ValidationResult)} Returns `undefined` if the validation of the duration succeeds. If the validation fails a `ValidationResult` object is returned.
   */
  this.book = function (callback) {
    if (!verifyExecution()) {
      return;
    }
    var data = ScreenModule.data;

    /** @type {DurationInput} */
    var durationInput = {
      duration: data.getCurrentDuration(),
      maxDuration: data.getMaxDuration(),
      minDuration: data.getMinDuration(),
      startTime: data.isFree() ? data.getCurrentTime() : data.getCurrent().PeriodDateTimeRange.Start,
    };

    var duration = durationInput.duration;
    var validationResult = validateTime(durationInput, callback);
    if (!validationResult.success) {
      return validationResult;
    }
    var input = { Type: "Book", Duration: duration };
    if (ScreenModule.data.isAreaScreen()) {
      //for area screen
      input.RoomId = ScreenModule.data.getFocusedRoom().RoomId;
    }
    $(document).trigger("screenDisplay:debug:book", [ScreenModule.data.getCurrentDuration()]);

    ScreenModule.data.bookClient();
    executeCommand(input, callback);
  };

  function validateTime(duration, callback) {
    var validationResult = screenApiValidation.validateBook(duration);
    if (!validationResult.success) {
      if (!callback) {
        ScreenModule.feedback.showInfo(validationResult.message);
      }
    }
    return validationResult;
  }

  this.cancel = function (id, callback) {
    if (!verifyExecution()) {
      return;
    }
    $(document).trigger("screenDisplay:debug:cancel", [""]);
    executeCommand({ Type: "Cancel", MeetingId: id }, callback);
  };
  this.confirm = function (meetingId, callback, noAlert) {
    if (!verifyExecution()) {
      return;
    }
    ScreenModule.data.setConfirmedOnscreen(meetingId);
    $(document).trigger("screenDisplay:debug:confirm", [""]);
    executeCommand({ Type: "Confirm", MeetingId: meetingId }, callback, noAlert);
  };
  this.confirmCurrent = function (callback) {
    if (!verifyExecution()) {
      return;
    }
    var meetingId = ScreenModule.data.getCurrent().MeetingId;
    ScreenModule.data.setConfirmedOnscreen(meetingId);
    executeCommand({ Type: "Confirm", MeetingId: meetingId }, callback);
  };
  this.confirmNext = function (callback) {
    if (!verifyExecution()) {
      return;
    }
    var meetingId = ScreenModule.data.getNext().MeetingId;
    ScreenModule.data.setConfirmedOnscreen(meetingId);
    executeCommand({ Type: "Confirm", MeetingId: meetingId }, callback);
  };
  this.endNow = function (callback) {
    if (!verifyExecution()) {
      return;
    }
    var input = { Type: "EndNow", MeetingId: ScreenModule.data.getCurrentId() };
    if (ScreenModule.data.isAreaScreen()) {
      // for area screen
      input.RoomId = ScreenModule.data.getFocusedRoom().RoomId;
    }
    endNowInternal(input, callback);
  };

  function endNowInternal(input, callback) {
    if (ScreenModule.data.isAreaScreen()) {
      //for area screen
      input.RoomId = ScreenModule.data.getFocusedRoom().RoomId;
      input.MeetingId = ScreenModule.data.getCurrentIdForRoom(input.RoomId);
    }
    $(document).trigger("screenDisplay:debug:end", [""]);
    ScreenModule.data.endNowClient();
    executeCommand(input, callback);
  }
  this.reschedule = function (callback) {
    if (!verifyExecution()) {
      return;
    }
    var duration = ScreenModule.data.getCurrentDuration();
    var input = {
      Type: "Reschedule",
      Duration: duration,
      MeetingId: ScreenModule.data.getCurrentId(),
    };
    if (ScreenModule.data.isAreaScreen()) {
      // for area screen
      input.RoomId = ScreenModule.data.getFocusedRoom().RoomId;
      input.MeetingId = ScreenModule.data.getCurrentIdForRoom(input.RoomId);
    }
    $(document).trigger("screenDisplay:debug:reschedule", [ScreenModule.data.getCurrentDuration()]);

    ScreenModule.data.rescheduleClient();
    executeCommand(input, callback);
  };

  function verifyExecution() {
    if (PRONESTORDISPLAY.isExecuting) {
      var feedbackData = {
        header: "",
        body: ScreenModule.settings.settings().localization.PleaseWaitAMoment,
      };
      ScreenModule.feedback.showMessage(feedbackData);
      return false;
    }
    return true;
  }
  //command
  function executeCommand(command, callback, noAlert) {
    PRONESTORDISPLAY.isExecuting = true;
    var securityToken = ScreenModule.helper.extractSecurityToken();
    var settings = ScreenModule.settings.settings();
    command = $.extend(
      {
        id: settings.interactivity.screenId,
        securityToken: securityToken,
        TimeStamp: ScreenModule.helper.getTimeStamp(),
      },
      command
    );
    ScreenModule.feedback.interactionIndication(command);
    ScreenModule.feedback.showLoading();
    $.when(post(command))
      .then(function (data) {
        if (callback) {
          callback(ScreenModule.feedback.wrapResult(data));
        } else {
          if (data.Status && data.Status.length > 0 && (!noAlert || !ScreenModule.data.wasConfirmed(data.Status))) {
            var feedbackData = {
              header: "Oops",
              body: data.Status,
            };
            ScreenModule.feedback.showMessage(feedbackData);
          }
          ScreenModule.settings.settings().interactionDone();
        }
      })
      .fail(function (jqXhr, textStatus, errorThrown) {
        if (callback) {
          callback(ScreenModule.feedback.wrapErrorResult(jqXhr, textStatus, errorThrown));
        } else {
          ScreenModule.feedback.showError(jqXhr, textStatus, errorThrown);
          ScreenModule.settings.settings().interactionDone();
        }
      })
      .always(function () {
        setTimeout(function () {
          PRONESTORDISPLAY.isExecuting = false;
        }, 1000);
      });
  }

  function post(postData) {
    PRONESTORDISPLAY.executingCommand = true;
    var dfd = $.Deferred();

    $.ajax({
      type: "POST",
      url: ScreenModule.settings.settings().interactivity.commandPath,
      data: postData,
      success: function (data) {
        dfd.resolve(data);
      },
      error: function (jqXhr, textStatus, errorThrown) {
        dfd.reject(jqXhr, textStatus, errorThrown);
      },
    });

    return dfd.promise();
  }

  function postDefect(postData, path) {
    PRONESTORDISPLAY.executingCommand = true;
    var dfd = $.Deferred();

    $.ajax({
      type: "POST",
      url: path,
      data: postData,
      success: function (data) {
        dfd.resolve(data);
      },
      error: function (jqXhr, textStatus, errorThrown) {
        dfd.reject(jqXhr, textStatus, errorThrown);
      },
    });

    return dfd.promise();
  }

  //error reporting
  this.reportDefect = function (defectOptionId, comment, callback) {
    var settings = ScreenModule.settings.settings();
    if (!comment) {
      comment = "";
    }
    var screenId;
    if (window.location.pathname.indexOf("?") !== -1) {
      screenId = window.location.pathname.substring(
        window.location.pathname.lastIndexOf("/") + 1,
        window.location.pathname.indexOf("?") - 1
      );
    } else {
      screenId = window.location.pathname.substring(window.location.pathname.lastIndexOf("/") + 1);
    }
    $.when(
      postDefect(
        {
          defectId: defectOptionId,
          displayScreenId: screenId,
          comment: comment,
        },
        settings.defect.reportDefectPath
      )
    )
      .then(function (data) {
        ScreenModule.settings.settings().interactionDone();
        //$.colorbox.close();
        if (callback) {
          callback();
        } else {
          console.log("show msg");
          if (ScreenModule.data.getDefect(defectOptionId).ConfirmationText) {
            var feedbackData = {
              header: ScreenModule.settings.settings().localization.ErrorReportSentHeader,
              body: ScreenModule.data.getDefect(defectOptionId).ConfirmationText,
            };
            ScreenModule.feedback.showMessage(feedbackData);
          }
        }
      })
      .fail(function (jqXhr, textStatus, errorThrown) {
        if (callback) {
          callback(ScreenModule.feedback.wrapErrorResult(jqXhr, textStatus, errorThrown));
        } else {
          ScreenModule.feedback.showError(jqXhr, textStatus, errorThrown);
          ScreenModule.settings.settings().interactionDone();
        }
      })
      .always(function () {});
  };

  this.clearDefect = function (defectOptionId, callback) {
    var settings = ScreenModule.settings.settings();
    var screenId;
    if (window.location.pathname.indexOf("?") !== -1) {
      screenId = window.location.pathname.substring(
        window.location.pathname.lastIndexOf("/") + 1,
        window.location.pathname.indexOf("?") - 1
      );
    } else {
      screenId = window.location.pathname.substring(window.location.pathname.lastIndexOf("/") + 1);
    }
    $.when(postDefect({ defectId: defectOptionId, displayScreenId: screenId }, settings.defect.clearDefectPath))
      .then(function (data) {
        ScreenModule.settings.settings().interactionDone();
        if (callback) {
          callback();
        } else {
          var feedbackData = {
            header: ScreenModule.settings.settings().localization.ErrorReportIssueSolvedHeader,
            body: ScreenModule.settings.settings().localization.ErrorReportIssueSolved,
          };
          ScreenModule.feedback.showMessage(feedbackData);
        }
      })
      .fail(function (jqXhr, textStatus, errorThrown) {
        if (callback) {
          callback(ScreenModule.feedback.wrapErrorResult(jqXhr, textStatus, errorThrown));
        } else {
          ScreenModule.feedback.showError(jqXhr, textStatus, errorThrown);
          ScreenModule.settings.settings().interactionDone();
        }
      })
      .always(function () {});
  };
}

function ScreenApiValidation() {
  // var screenData = new ScreenData();

  /**
   * Validates the duration. Called before booking a room.
   * @param {DurationInput} durationInput - Duration to be validated.
   * @returns {ValidationResult} Validation result.
   */
  this.validateBook = function (durationInput) {
    var validationResult = {};
    var durationLeft = ScreenModule.time.getDiffMinutes(ScreenModule.data.getCurrentTime(), dateValue);
    validationResult.success = true;
    if (durationLeft > durationInput.maxDuration) {
      validationResult.message = "Cannot book more than " + durationInput.maxDuration;
      validationResult.success = false;
    }
    if (durationLeft < durationInput.minDuration) {
      validationResult.message = "Cannot book less than " + durationInput.minDuration;
      validationResult.success = false;
    }

    if (validationResult.success) {
      var dateValue = ScreenModule.time.addToTime(durationInput.startTime, durationInput.duration);
      dateValue = ScreenModule.duration.roundUpToNearesQuarter(dateValue);
      if (durationLeft < 0) {
        validationResult.message = "Cannot book next day";
        validationResult.success = false;
      }
    }

    return validationResult;
  };
}
